@@ -17,34 +17,30 @@
  */
 
 #include "config.h"
-#define GDK_PIXBUF_ENABLE_BACKEND /* Ugly? */
 #include "gdkdisplay.h"
 #include "gdkcursor.h"
 #include "gdkwin32.h"
 #include "gdktextureprivate.h"
-#include <glib/gi18n-lib.h>
 
 #include "gdkdisplay-win32.h"
 
-#ifdef __MINGW32__
-#include <w32api.h>
-#endif
-
 #include "xcursors.h"
 
-typedef struct _DefaultCursor {
+#include <stdint.h>
+
+static struct {
   char *name;
   char *id;
-} DefaultCursor;
-
-static DefaultCursor default_cursors[] = {
+} default_cursors[] = {
+  /* -- Win32 cursor names: -- */
   { "appstarting", IDC_APPSTARTING },
   { "arrow", IDC_ARROW },
   { "cross", IDC_CROSS },
+  { "dnd-move", IDC_ARROW },
   { "hand",  IDC_HAND },
   { "help",  IDC_HELP },
   { "ibeam", IDC_IBEAM },
-  /* an X cursor name, for compatibility with GTK: */
+  /* -- X11 cursor names: -- */
   { "left_ptr_watch", IDC_APPSTARTING },
   { "sizeall", IDC_SIZEALL },
   { "sizenesw", IDC_SIZENESW },
@@ -53,7 +49,7 @@ static DefaultCursor default_cursors[] = {
   { "sizewe", IDC_SIZEWE },
   { "uparrow", IDC_UPARROW },
   { "wait", IDC_WAIT },
-  /* css cursor names: */
+  /* -- CSS cursor names: -- */
   { "default", IDC_ARROW },
   { "pointer", IDC_HAND },
   { "progress", IDC_APPSTARTING },
@@ -93,18 +89,17 @@ struct _GdkWin32HCursor
 
   /* Do not do any modifications to the handle
    * (i.e. do not call DestroyCursor() on it).
-   * It's a "read-only" copy, the original is stored
-   * in the display instance.
-   */
+   * It's a "read-only" copy, the original is
+   * stored in the display instance */
   HANDLE           readonly_handle;
 
-  /* This is a way to access the real handle stored
-   * in the display.
-   * TODO: make it a weak reference
-   */
+  /* This is a way to access the real handle
+   * stored in the display.
+   * TODO: make it a weak reference */
   GdkWin32Display *display;
 
-  /* A copy of the "destoyable" attribute of the handle */
+  /* A copy of the "destoyable" attribute of
+   * the handle */
   gboolean         readonly_destroyable;
 };
 
@@ -382,7 +377,8 @@ hcursor_from_x_cursor (int          i,
   int j, x, y, ofs;
   HCURSOR rv;
   int w, h;
-  guchar *and_plane, *xor_plane;
+  uint8_t *and_plane;
+  uint8_t *xor_plane;
 
   w = GetSystemMetrics (SM_CXCURSOR);
   h = GetSystemMetrics (SM_CYCURSOR);
@@ -406,7 +402,7 @@ hcursor_from_x_cursor (int          i,
 	  for (x = 0; x < cursors[i].width && x < w ; x++, j++)
 	    {
 	      int pofs = ofs + x / 8;
-	      guchar data = (cursors[i].data[j/4] & (0xc0 >> (2 * (j%4)))) >> (2 * (3 - (j%4)));
+              uint8_t data = (cursors[i].data[j/4] & (0xc0 >> (2 * (j%4)))) >> (2 * (3 - (j%4)));
 	      int bit = 7 - (j % cursors[i].width) % 8;
 
 	      if (data)
@@ -422,13 +418,12 @@ hcursor_from_x_cursor (int          i,
 #undef SET_BIT
 #undef RESET_BIT
 
-      rv = CreateCursor (_gdk_app_hmodule, cursors[i].hotx, cursors[i].hoty,
+      rv = CreateCursor (NULL, cursors[i].hotx, cursors[i].hoty,
 			 w, h, and_plane, xor_plane);
     }
   else
     {
-      rv = CreateCursor (_gdk_app_hmodule, 0, 0,
-			 w, h, and_plane, xor_plane);
+      rv = CreateCursor (NULL, 0, 0, w, h, and_plane, xor_plane);
     }
 
   if (rv == NULL)
@@ -471,7 +466,7 @@ win32_cursor_create_win32hcursor (GdkWin32Display *display,
         break;
       case GDK_WIN32_CURSOR_LOAD_FROM_RESOURCE_THIS:
         result = gdk_win32_hcursor_new (display,
-                                        LoadImageA (_gdk_app_hmodule,
+                                        LoadImageA (GetModuleHandle (NULL),
                                                     (const char *) cursor->resource_name,
                                                     IMAGE_CURSOR,
                                                     cursor->width,
@@ -481,7 +476,7 @@ win32_cursor_create_win32hcursor (GdkWin32Display *display,
         break;
       case GDK_WIN32_CURSOR_LOAD_FROM_RESOURCE_GTK:
         result = gdk_win32_hcursor_new (display,
-                                        LoadImageA (_gdk_dll_hinstance,
+                                        LoadImageA (this_module (),
                                                     (const char *) cursor->resource_name,
                                                     IMAGE_CURSOR,
                                                     cursor->width,
@@ -852,7 +847,7 @@ static GdkWin32HCursor *
 create_blank_win32hcursor (GdkWin32Display *display)
 {
   int w, h;
-  guchar *and_plane, *xor_plane;
+  uint8_t *and_plane, *xor_plane;
   HCURSOR rv;
 
   w = GetSystemMetrics (SM_CXCURSOR);
@@ -863,9 +858,7 @@ create_blank_win32hcursor (GdkWin32Display *display)
   xor_plane = g_malloc ((w/8) * h);
   memset (xor_plane, 0, (w/8) * h);
 
-  rv = CreateCursor (_gdk_app_hmodule, 0, 0,
-                     w, h, and_plane, xor_plane);
-
+  rv = CreateCursor (NULL, 0, 0, w, h, and_plane, xor_plane);
   if (rv == NULL)
     WIN32_API_FAILED ("CreateCursor");
 
@@ -876,6 +869,7 @@ static GdkWin32HCursor *
 gdk_win32hcursor_create_for_name (GdkWin32Display  *display,
                                   const char *name)
 {
+  const HINSTANCE hinstance = GetModuleHandle (NULL);
   GdkWin32HCursor *win32hcursor = NULL;
 
   /* Blank cursor case */
@@ -890,7 +884,7 @@ gdk_win32hcursor_create_for_name (GdkWin32Display  *display,
   /* Allow to load named cursor resources linked into the executable.
    * Cursors obtained with LoadCursor() cannot be destroyed.
    */
-  return gdk_win32_hcursor_new (display, LoadCursor (_gdk_app_hmodule, name), FALSE);
+  return gdk_win32_hcursor_new (display, LoadCursor (hinstance, name), FALSE);
 }
 
 static HICON
@@ -899,11 +893,11 @@ pixbuf_to_hicon (GdkPixbuf *pixbuf,
                  int        x,
                  int        y);
 
-static GdkWin32HCursor *
-gdk_win32hcursor_create_for_texture (GdkWin32Display *display,
-                                     GdkTexture      *texture,
-                                     int              x,
-                                     int              y)
+HICON
+_gdk_win32_create_hicon_for_texture (GdkTexture *texture,
+                                     gboolean    is_icon,
+                                     int         x,
+                                     int         y)
 {
   cairo_surface_t *surface;
   GdkPixbuf *pixbuf;
@@ -913,13 +907,26 @@ gdk_win32hcursor_create_for_texture (GdkWin32Display *display,
   surface = gdk_texture_download_surface (texture);
   width = cairo_image_surface_get_width (surface);
   height = cairo_image_surface_get_height (surface);
-  
+
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   pixbuf = gdk_pixbuf_get_from_surface (surface, 0, 0, width, height);
-  
-  icon = pixbuf_to_hicon (pixbuf, TRUE, 0, 0);
-  
+G_GNUC_END_IGNORE_DEPRECATIONS
+
+  icon = pixbuf_to_hicon (pixbuf, is_icon, x, y);
+
   g_object_unref (pixbuf);
-  
+
+  return icon;
+}
+
+static GdkWin32HCursor *
+gdk_win32hcursor_create_for_texture (GdkWin32Display *display,
+                                     GdkTexture      *texture,
+                                     int              x,
+                                     int              y)
+{
+  HICON icon = _gdk_win32_create_hicon_for_texture (texture, FALSE, x, y);
+
   return gdk_win32_hcursor_new (display, (HCURSOR) icon, TRUE);
 }
 
@@ -1009,8 +1016,9 @@ gdk_win32_icon_to_pixbuf_libgtk_only (HICON hicon,
     RGBQUAD colors[2];
   } bmi;
   HDC hdc;
-  guchar *pixels, *bits;
-  int rowstride, x, y, w, h;
+  uint8_t *pixels, *bits;
+  int x, y, w, h;
+  gsize rowstride;
 
   if (!GDI_CALL (GetIconInfo, (hicon, &ii)))
     return NULL;
@@ -1062,7 +1070,7 @@ gdk_win32_icon_to_pixbuf_libgtk_only (HICON hicon,
 		no_alpha = FALSE;
 	      pixels += 4;
 	    }
-	  pixels += (w * 4 - rowstride);
+	  pixels += rowstride - w * 4;
 	}
 
       /* mask */
@@ -1077,7 +1085,7 @@ gdk_win32_icon_to_pixbuf_libgtk_only (HICON hicon,
 		  pixels[3] = 255 - bits[(x + y * w) * 4];
 		  pixels += 4;
 		}
-	      pixels += (w * 4 - rowstride);
+	      pixels += rowstride - w * 4;
 	    }
 	}
     }
@@ -1117,7 +1125,7 @@ gdk_win32_icon_to_pixbuf_libgtk_only (HICON hicon,
 
       for (y = 0; y < h; y++)
 	{
-	  const guchar *andp, *xorp;
+          const uint8_t *andp, *xorp;
 	  if (bmi.bi.biHeight < 0)
 	    {
 	      andp = bits + bpl*y;
@@ -1151,7 +1159,7 @@ gdk_win32_icon_to_pixbuf_libgtk_only (HICON hicon,
 		  xorp++;
 		}
 	    }
-	  pixels += (w * 4 - rowstride);
+	  pixels += rowstride - w * 4;
 	}
     }
 
@@ -1178,8 +1186,8 @@ gdk_win32_icon_to_pixbuf_libgtk_only (HICON hicon,
  */
 
 static HBITMAP
-create_alpha_bitmap (int      size,
-		     guchar **outdata)
+create_alpha_bitmap (int       size,
+                     uint8_t **outdata)
 {
   BITMAPV5HEADER bi;
   HDC hdc;
@@ -1216,9 +1224,9 @@ create_alpha_bitmap (int      size,
 }
 
 static HBITMAP
-create_color_bitmap (int      size,
-		     guchar **outdata,
-		     int      bits)
+create_color_bitmap (int       size,
+                     uint8_t **outdata,
+                     int       bits)
 {
   struct {
     BITMAPV4HEADER bmiHeader;
@@ -1265,8 +1273,9 @@ pixbuf_to_hbitmaps_alpha_winxp (GdkPixbuf *pixbuf,
    * http://www.dotnet247.com/247reference/msgs/13/66301.aspx
    */
   HBITMAP hColorBitmap, hMaskBitmap;
-  guchar *indata, *inrow;
-  guchar *colordata, *colorrow, *maskdata, *maskbyte;
+  const uint8_t *indata;
+  const uint8_t *inrow;
+  uint8_t *colordata, *colorrow, *maskdata, *maskbyte;
   int width, height, size, i, i_offset, j, j_offset, rowstride;
   guint maskstride, mask_bit;
 
@@ -1289,7 +1298,7 @@ pixbuf_to_hbitmaps_alpha_winxp (GdkPixbuf *pixbuf,
   /* MSDN says mask rows are aligned to "LONG" boundaries */
   maskstride = (((size + 31) & ~31) >> 3);
 
-  indata = gdk_pixbuf_get_pixels (pixbuf);
+  indata = gdk_pixbuf_read_pixels (pixbuf);
   rowstride = gdk_pixbuf_get_rowstride (pixbuf);
 
   if (width > height)
@@ -1343,8 +1352,9 @@ pixbuf_to_hbitmaps_normal (GdkPixbuf *pixbuf,
    * http://www.dotnet247.com/247reference/msgs/13/66301.aspx
    */
   HBITMAP hColorBitmap, hMaskBitmap;
-  guchar *indata, *inrow;
-  guchar *colordata, *colorrow, *maskdata, *maskbyte;
+  const uint8_t *indata;
+  const uint8_t *inrow;
+  uint8_t *colordata, *colorrow, *maskdata, *maskbyte;
   int width, height, size, i, i_offset, j, j_offset, rowstride, nc, bmstride;
   gboolean has_alpha;
   guint maskstride, mask_bit;
@@ -1373,7 +1383,7 @@ pixbuf_to_hbitmaps_normal (GdkPixbuf *pixbuf,
   /* MSDN says mask rows are aligned to "LONG" boundaries */
   maskstride = (((size + 31) & ~31) >> 3);
 
-  indata = gdk_pixbuf_get_pixels (pixbuf);
+  indata = gdk_pixbuf_read_pixels (pixbuf);
   rowstride = gdk_pixbuf_get_rowstride (pixbuf);
   nc = gdk_pixbuf_get_n_channels (pixbuf);
   has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
@@ -1454,37 +1464,6 @@ pixbuf_to_hicon (GdkPixbuf *pixbuf,
   return icon;
 }
 
-HICON
-_gdk_win32_texture_to_hicon (GdkTexture *texture)
-{
-  cairo_surface_t *surface;
-  GdkPixbuf *pixbuf;
-  int width, height;
-  HICON icon;
-  
-  surface = gdk_texture_download_surface (texture);
-  width = cairo_image_surface_get_width (surface);
-  height = cairo_image_surface_get_height (surface);
-  
-  pixbuf = gdk_pixbuf_get_from_surface (surface, 0, 0, width, height);
-  
-  icon = pixbuf_to_hicon (pixbuf, TRUE, 0, 0);
-  
-  g_object_unref (pixbuf);
-  
-  return icon;
-}
-
-HICON
-_gdk_win32_pixbuf_to_hcursor (GdkPixbuf *pixbuf,
-			      int        x_hotspot,
-			      int        y_hotspot)
-{
-  return pixbuf_to_hicon (pixbuf, FALSE, x_hotspot, y_hotspot);
-}
-
-
-
 /**
  * gdk_win32_display_get_win32hcursor:
  * @display: (type GdkWin32Display): a `GdkDisplay`
